#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

/// リミッタマクロ thには正の数を指定
#define LIMIT(x,th) ((x>th) ? th : ((x<-th) ? -th : x))

/// 矢印描画のためのパラメータ
#define ARROW_L (10)
#define ARROW_W (5)

static int press_flag = 0;
static double horizon;
static double vertical;

/** 描画実体
 */
void
draw(GtkWidget *widget, cairo_t *cr)
{
  double x0 = widget->allocation.x; /// ウィンドウの左上座標x
  double y0 = widget->allocation.y; /// ウィンドウの左上座標y
  double width = widget->allocation.width;   /// ウィンドウの幅
  double height = widget->allocation.height; /// ウィンドウの高さ
  double xc = x0 + width / 2.0;  /// ウィンドウの中心座標
  double yc = y0 + height / 2.0; /// ウィンドウの中心座標

  /// 線の色を設定
  cairo_set_source_rgb(cr, 1, 0, 0);
  /// X軸線
  cairo_move_to(cr, x0, yc);
  cairo_line_to(cr, x0+width, yc);
  cairo_line_to(cr, x0+width-ARROW_L, yc-ARROW_W);
  cairo_move_to(cr, x0+width, yc);
  cairo_line_to(cr, x0+width-ARROW_L, yc+ARROW_W);
  /// Y軸線
  cairo_move_to(cr, xc, y0+height);
  cairo_line_to(cr, xc, y0);
  cairo_line_to(cr, xc+ARROW_W, y0+ARROW_L);
  cairo_move_to(cr, xc, y0);
  cairo_line_to(cr, xc-ARROW_W, y0+ARROW_L);
  /// 全描画
  cairo_stroke(cr);
}

/** cairoを利用した背景画像の描画
 */
void
background_image_expose(GtkWidget       *widget,
			GdkEventExpose  *event,
			gpointer         user_data)
{
  cairo_t *cr;

  /// cairoオブジェクトの作成
  cr = gdk_cairo_create(widget->window);

  /// 描画する
  draw(widget, cr);
  
  /// cairoオブジェクトの削除
  cairo_destroy(cr);
}

void
on_window1_destroy                     (GtkObject       *object,
                                        gpointer         user_data)
{
  gtk_main_quit();
}


gboolean
on_drawingarea1_button_press_event     (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  press_flag = 1;
  return FALSE;
}


gboolean
on_drawingarea1_button_release_event   (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  press_flag = 0;
  horizon  = 0.0;
  vertical = 0.0;
  fprintf(stderr, "(%.2f, %.2f)\n", horizon, vertical);
  return FALSE;
}


gboolean
on_drawingarea1_motion_notify_event    (GtkWidget       *widget,
                                        GdkEventMotion  *event,
                                        gpointer         user_data)
{
  int mouse_x, mouse_y;/// マウス座標
  double x0 = widget->allocation.x; /// ウィンドウの左上座標x
  double y0 = widget->allocation.y; /// ウィンドウの左上座標y
  double width = widget->allocation.width;   /// ウィンドウの幅
  double height = widget->allocation.height; /// ウィンドウの高さ
  double xc = x0 + width / 2.0;  /// ウィンドウの中心座標
  double yc = y0 + height / 2.0; /// ウィンドウの中心座標
  double gain_x = (width  - 1)/2;
  double gain_y = (height - 1)/2;
  double rate_x, rate_y; /// 比率 -1〜1

  /// マウスポインタ座標を受け取る
  mouse_x = (int)event->x;
  mouse_y = (int)event->y;

  /// ウィンドウ中心を原点としてマウス位置を-1〜1で表す
  rate_x = (mouse_x - xc) / gain_x;
  rate_y = (yc - mouse_y) / gain_y;
  rate_x = LIMIT(rate_x, 1.0);
  rate_y = LIMIT(rate_y, 1.0);

  if(press_flag==1){
    horizon  = rate_x;
    vertical = rate_y;
    fprintf(stderr, "(%.2f, %.2f)\n", horizon, vertical);
  }
  
  return FALSE;
}


gboolean
on_drawingarea1_expose_event           (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
  /// 背景の座標軸を描画
  background_image_expose(widget, event, user_data);
  return FALSE;
}

